// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

void sim_celiv(struct device *in)
{
//<clean>
in->dt=in->celiv_dt;

carrier_count_reset(in);

in->go_time=FALSE;


//struct istruct celivout;
//inter_init(&celivout);
in->externalv=in->celiv_Vexternal;
printf("Running celiv. %e\n",in->externalv);
int ittr=0;

int step=0;
double dj;
double j0;
solve_light(in,&(in->mylight),in->Psun,0.0);

//in->Vapplied=0.1;
//ramp(in,in->Vapplied,in->celiv_Vmax,100.0);
//sim_externalv(in,in->celiv_Vmax);
//double celiv_max=in->celiv_Vmax;

//ramp(in,in->Vapplied,in->celiv_Vexternal,100.0);
sim_externalv(in,in->celiv_Vexternal);
in->Vapplied=in->celiv_Vexternal;
double V=in->Vapplied;
double V0=in->celiv_Vexternal;
solve_light(in,&(in->mylight),in->Psun,0.0);

newton_celiv(in,V0,FALSE);
device_timestep(in);
newton_celiv(in,V0,FALSE);
device_timestep(in);

//printf("\n\n%le\n",in->Vapplied_last);
//exit(0);

//printf("Vapplied_last %le %le\n",in->Vapplied,in->Vapplied_last);

time_init(in);

device_timestep(in);

in->go_time=TRUE;


in->time=0.0;
//double np_out_tot=0.0;
double t0=in->celiv_t0;
double t1=in->celiv_t0+in->celiv_t1;
double t2=in->celiv_t0+in->celiv_t1+in->celiv_t2;
FILE *celiv_out=fopen("./celiv_j.dat","w");
fclose(celiv_out);

celiv_out=fopen("./celiv_v.dat","w");
fclose(celiv_out);

celiv_out=fopen("./celiv_nleft.dat","w");
fclose(celiv_out);

celiv_out=fopen("./celiv_rtot.dat","w");
fclose(celiv_out);

double ramp_rate=in->celiv_Vmax*(in->dt/(t1-t0));
double local_time=in->time;
double i_max=0.0;
double t_max=0.0;
double mu=0.0;
double A=fabs(in->celiv_Vmax/(t1-t0));
double np_den=0.0;
struct istruct out_j;
inter_init(&out_j);

struct istruct out_v;
inter_init(&out_v);

struct istruct out_mue;
inter_init(&out_mue);

struct istruct out_muh;
inter_init(&out_muh);

struct istruct out_mue_c;
inter_init(&out_mue_c);

struct istruct out_muh_c;
inter_init(&out_muh_c);

struct istruct out_G;
inter_init(&out_G);

struct istruct lost_charge;
inter_init(&lost_charge);

double mu_e_start=get_avg_mue(in);
double mu_h_start=get_avg_muh(in);

//	printf("%le %le\n",in->Vapplied,in->Vapplied_last);
	//exit(0);
double extracted_through_contacts=0.0;
//reset_npinit(in);
do
{


	if (local_time<t0)
	{
		V=V0;

		if ((local_time>=in->celiv_laser_on)&&(local_time<=(in->celiv_laser_on+in->celiv_laser_width)))
		{
			solve_light(in,&(in->mylight),in->Psun,in->celiv_laser_power);
		}else
		{
			solve_light(in,&(in->mylight),in->Psun,0.0);
		}

		extracted_through_contacts=0.0;
	}else
	if (local_time<t1)
	{
		if (in->celiv_Vss==0)
		{
			V+=ramp_rate;
		}else
		{
			V=in->celiv_Vss;
		}
		celiv_out=fopen("./celiv_nleft.dat","a");
		fprintf(celiv_out,"%le %le\n",in->time,get_extracted_np(in));
		fclose(celiv_out);

		//light_update(in,in->toflaserpower);
	}else
	{
		V=V0;
	}


	//solve_all(in);

	double i0=-newton_celiv(in,V,TRUE);

	if ((local_time>t0)&&(local_time<t1))
	{
		//printf("%le %le\n",i_max,i0);
		//getchar();
		if (i0>i_max)
		{
			i_max=i0;
			t_max=local_time-t0;
		}
		j0=fabs(in->C*(in->Vapplied-in->Vapplied_last)/in->dt);
		extracted_through_contacts+=-in->dt*get_J(in)/in->ylen/q;
	}

	if (local_time<t0)
	{
		np_den=fabs(get_extracted_n(in)+get_extracted_p(in))/2.0;
		//printf("%le\n",np_den);
		//getchar();
	}

	#ifndef mindump
	printf("celiv time=%e step=%d %.1e ",in->time,step,in->last_error);
	char out[100];
	sprintf(out,"Vtot=%lf current = %e mA (%e A/m^2)\n",V,get_I(in)/1e-3,get_J(in));

	printdebug(out);
	#endif

	ittr++;

	plot_now(in,"plot");

	//inter_append(&celivout,in->time,fabs(get_I(in)));

	inter_append(&out_j,in->time,i0);
	inter_append(&out_v,in->time,V);
	inter_append(&out_mue,in->time,get_avg_mue(in));
	inter_append(&out_muh,in->time,get_avg_muh(in));
	inter_append(&out_mue_c,in->time,in->mun[0]);
	inter_append(&out_muh_c,in->time,in->mup[0]);
	inter_append(&out_G,in->time,in->Gn[0]);
	inter_append(&lost_charge,in->time,extracted_through_contacts-fabs(get_extracted_n(in)+get_extracted_p(in))/2.0);

	device_timestep(in);
	step++;

	if (in->time>(((double)in->celiv_repeat)*t2)) break;
	if (local_time>t2) local_time=0.0;
	local_time+=in->dt;

	if (in->stop==TRUE)
	{
		break;

	}


}while(1);
char outpath[1000];
sprintf(outpath,"%s%s",in->outputpath,"celiv_j.dat");
inter_save(&out_j,outpath);


sprintf(outpath,"%s%s",in->outputpath,"celiv_v.dat");
inter_save(&out_v,outpath);

sprintf(outpath,"%s%s",in->outputpath,"celiv_mue.dat");
inter_save(&out_mue,outpath);

sprintf(outpath,"%s%s",in->outputpath,"celiv_muh.dat");
inter_save(&out_muh,outpath);

sprintf(outpath,"%s%s",in->outputpath,"celiv_mue_c.dat");
inter_save(&out_mue_c,outpath);

sprintf(outpath,"%s%s",in->outputpath,"celiv_muh_c.dat");
inter_save(&out_muh_c,outpath);

sprintf(outpath,"%s%s",in->outputpath,"celiv_G.dat");
inter_save(&out_G,outpath);

sprintf(outpath,"%s%s",in->outputpath,"celiv_lost_charge.dat");
inter_save(&lost_charge,outpath);

printf("celiv stats\n");
//inter_get(&out_j,in->celiv_time_j1)-inter_get(&out_j,in->celiv_time_j0)

dj=fabs(inter_get(&out_j,in->celiv_time_j1)-i_max);


mu=(2.0/3.0)*in->ylen*in->ylen/(A*t_max*t_max*(1.0+0.21*(dj/j0)));
printf("%le %le\n",in->ylen*in->ylen,(A*t_max*t_max*(1.0+0.21*(dj/j0))));
printf("t_max=%le i_max=%le mu=%le dj=%le j0=%le\n",t_max,i_max,mu,dj,j0);
celiv_out=fopen("./celiv_mu_value.dat","w");
fprintf(celiv_out,"%le %le %le\n",mu_e_start,mu_h_start,mu);
fclose(celiv_out);

celiv_out=fopen("./celiv_ce_eff.dat","w");
fprintf(celiv_out,"%le %le %le %le %le\n",in->celiv_Vmax,in->celiv_laser_power,(extracted_through_contacts/np_den)*100.0,extracted_through_contacts,np_den);
fclose(celiv_out);

in->go_time=FALSE;

inter_free(&out_G);
inter_free(&out_muh);
inter_free(&out_mue);
inter_free(&out_muh_c);
inter_free(&out_mue_c);
inter_free(&out_j);
inter_free(&out_v);
//</clean>
}
