// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>


void sim_simple_jv(struct device *in)
{
double V=in->Vstart;
double Vstop=in->Vstop;
double Vstep=in->Vstep;
FILE *iv;
int ittr=0;
double P;
int first=TRUE;
double Vlast;
double Ilast;
double Plast;
double Vexternal;
struct istruct ivexternal;
inter_init(&ivexternal);

struct istruct jvexternal;
inter_init(&jvexternal);

double Isc=0.0;
double Iout=0.0;

do
{
//exp((q*V)/(4.1400e-21*in->n_id))
	double I_diode=in->Is*(exp((q*V)/(4.1400e-21*in->n_id)))-in->Igen;
	double Iout=I_diode+V/in->Rshunt;
	double J=Iout/in->A;
	Vexternal=Iout*in->Rcontact+V;

	printf("V=%lf %lf current = %le mA\n",V,Vexternal,I/1e-3);

	inter_append(&ivexternal,Vexternal,Iout);
	inter_append(&jvexternal,Vexternal,J);
	ittr++;

	P=fabs(Iout*Vexternal);
	if (first==FALSE)
	{

		//check if we have crossed 0V
		if ((Vlast<=0)&&(V>=0.0))
		{
			Isc=(Ilast+(Iout-Ilast)*(0-Vlast)/(V-Vlast));
			in->Jsc=Isc/in->A;
			printf("Jsc = %e\n",in->Jsc);
		}

		if ((Ilast<=0)&&(Iout>=0.0))
		{
			in->Voc=Vlast+(Vexternal-Vlast)*(0-Ilast)/(Iout-Ilast);
			printf("Voc = %e\n",in->Voc);
		}

		if ((P>Plast)&&(Vexternal>0.0)&&(Iout<0.0))
		{
			in->Pmax=P;
			in->Pmax_voltage=Vexternal;
		}

		if (Vexternal>Vstop)
		{
			printf("one\n");
			break;
		}

	}



	Ilast=I;
	Vlast=Vexternal;
	Plast=P;
	first=FALSE;

	V+=Vstep;

	if ((Vstep>=0)&&(V>Vstop))
	{
		in->stop=TRUE;
		printf("1111\n");
	}

	if ((Vstep<0)&&(V<Vstop))
	{
		in->stop=TRUE;
		printf("2222\n");
	}


	if (in->stop==TRUE)
	{
		break;
	}



}while(1);

in->FF=fabs(in->Pmax/(Isc*in->Voc));

char out[200];
sprintf(out,"Voc= %lf (V)\n",in->Voc);
printdebug( out);
sprintf(out,"Isc= %lf (A)\n",Isc);
printdebug( out);
sprintf(out,"Pmax= %lf (W/m^2)\n",in->Pmax);
printdebug( out);
sprintf(out,"Voltage to get Pmax= %lf (V)\n",in->Pmax_voltage);
printdebug( out);
sprintf(out,"FF= %lf\n",in->FF*100.0);
printdebug( out);

sprintf(out,"Efficiency= %lf percent\n",mod((in->Pmax/in->A)/in->Psun)*100.0);
printdebug( out);

iv=fopen("./ff.dat","w");
fprintf(iv,"%le\n",in->FF);
fclose(iv);

FILE *outfile=fopen("./eff.dat","w");
fprintf(outfile,"%le",mod((in->Pmax/in->A)/in->Psun)*100.0);
fclose(outfile);

outfile=fopen("./voc.dat","w");
fprintf(outfile,"%le",in->Voc);
fclose(outfile);

outfile=fopen("./sc.dat","w");
fprintf(outfile,"%le",in->Jsc);
fclose(outfile);

inter_save_a(&ivexternal,in->outputpath,"ivexternal.dat");
inter_save_a(&jvexternal,in->outputpath,"jvexternal.dat");
inter_free(&ivexternal);
inter_free(&jvexternal);
}

