// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "util.h"
#include "dump.h"
#include "true_false.h"

double read_value(char *file,int skip,int line)
{
FILE *in;
char buf[1000];
double value;
in=fopen(file,"r");
int l=0;

do
{
l++;
	fscanf(in,"%s",buf);

	if (l==line)
	{
		sscanf((buf+skip),"%le\n",&value);
		break;
	}


}while (!feof(in));

fclose(in);


return value;
}

void safe_file(char *name)
{
FILE *file;
file = fopen(name, "rb");

if (!file)
{
	printf("File %s not found\n",name);
	getchar();
	exit(0);
}

fclose(file);
}

int cmpstr_min(char * in1,char *in2)
{
int i;
int max=strlen(in1);
if (strlen(in2)<max) max=strlen(in2);
for (i=0;i<max;i++)
{
	if (in1[i]!=in2[i]) return 1;
}
return 0;
}

int strcmp_end(char * str,char *end)
{

if (strlen(str)<strlen(end)) return 1;
int pos=strlen(str)-strlen(end);
//printf("cmp=%s %s\n",str+pos,end);
return strcmp((char *)(str+pos),end);
}

int extract_str_number(char * in,char *cut)
{
int out;
int len=strlen(cut);
sscanf((in+len),"%d",&out);
return out;
}

void waveprint(char *in,double wavelength)
{
if (wavelength<400.0)
{
	textcolor(fg_purple);
}else
if (wavelength<500.0)
{
	textcolor(fg_blue);
}else
if (wavelength<575.0)
{
	textcolor(fg_green);
}else
if (wavelength<600.0)
{
	textcolor(fg_yellow);
}else
{
	textcolor(fg_red);
}

printf("%s",in);
textcolor(fg_reset);

}

void randomprint(char *in)
{
if (get_dump_status(dump_print_text)==TRUE)
{
	int i;
	for (i=0;i<strlen(in);i++)
	{
	int rnd=(float)6.0*rand()/(float)RAND_MAX;
		if (rnd==0) textcolor(fg_reset);
		if (rnd==1) textcolor(fg_red);
		if (rnd==2) textcolor(fg_green);
		if (rnd==3) textcolor(fg_yellow);
		if (rnd==4) textcolor(fg_blue);
		if (rnd==5) textcolor(fg_purple);
		printf("%c",in[i]);
	}
}else
{
	printf("%s",in);
}

}

void textcolor(int color)
{	char command[13];

	/* Command is the control command to the terminal */
	if (get_dump_status(dump_print_text)==TRUE)
	{
	sprintf(command, "\e[%dm", color);
	printf("%s", command);
	}
}

FILE *fopena(char *path,char *name,const char *mode)
{
char wholename[200];
strcpy(wholename,path);
strcat(wholename,name);
FILE *pointer;
pointer=fopen(wholename,mode);
//if (pointer == NULL)
//{
//	printf("no file %s\n",wholename);
//	exit(0);
//}
return pointer;
}

int scanarg( char* in[],int count,char * find)
{
int i;
for (i=0;i<count;i++)
{
if (strcmp(in[i],find)==0) return TRUE;
}
return FALSE;
}

int get_arg_plusone_pos( char* in[],int count,char * find)
{
int i;
for (i=0;i<count;i++)
{
if (strcmp(in[i],find)==0)
{
       if ((i+1)<count)
       {
               return i+1;
       }else
       {
               return FALSE;
       }
}
}
return FALSE;
}

char * get_arg_plusone( char* in[],int count,char * find)
{
int i;
static char no[] = "";
for (i=0;i<count;i++)
{
//printf("%s %s\n",in[i],find);
if (strcmp(in[i],find)==0)
{
       if ((i+1)<count)
       {
		//printf("%s\n",in[i+1]);
               return in[i+1];
       }else
       {
               return no;
       }
}
}

return no;
}
